# Copyright (c) 2008-2009 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import martian
from martian import priority
from martian import component
from martian import baseclass
from martian.directive import StoreMultipleTimes
from zope.interface.interface import TAGGED_DATA

class TaggedValueStoreMutipleTimes(StoreMultipleTimes):
    """Stores the directive value in a interface tagged value.
    """

    def get(self, directive, component, default):
        return component.queryTaggedValue(directive.dotted_name(), default)

    def set(self, locals_, directive, value):
        taggeddata = locals_.setdefault(TAGGED_DATA, {})
        if directive.dotted_name() in taggeddata:
            taggeddata[directive.dotted_name()].append(value)
        else:
            taggeddata[directive.dotted_name()] = [value, ]

    def setattr(self, context, directive, value):
        context.setTaggedValue(directive.dotted_name(), [value, ])


class resource(martian.Directive):
    scope = martian.CLASS
    store = TaggedValueStoreMutipleTimes()

from grokcore.component import name
from grokcore.component import title
from grokcore.component import context
from grokcore.view import skin
from grokcore.view import layer
from grokcore.viewlet import view
from grokcore.viewlet import viewletmanager
from grokcore.viewlet import order




# Copyright (c) 2002-2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from AccessControl.PermissionRole import PermissionRole
from App.ProductContext import AttrDict
from App.FactoryDispatcher import FactoryDispatcher
from Interface.Implements import instancesOfObjectImplements
from Products.Five.fiveconfigure import unregisterClass
from OFS import misc_ as icons

from zope.configuration.name import resolve
from zope.interface import implementedBy

import AccessControl.Permission
import Globals
import Products

from Products.Silva import mangle
from Products.Silva import interfaces as silvainterfaces
from Products.Silva.icon import registry as icon_registry
from Products.Silva.helpers import add_and_edit, makeContainerFilter
from Products.Silva.ExtensionRegistry import extensionRegistry

import os.path

# Views utils

def getFiveViewNameFor(context):
    """Return the correct name for the view if you want to use a Five
    default one.
    """
    stack = [silvainterfaces.IContainer,
             silvainterfaces.IAsset,
             silvainterfaces.IVersionedContent,
             silvainterfaces.IContent]

    for interface in stack:
        if interface.providedBy(context):
            return 'Five %s' % interface.__name__[1:]


def getSilvaViewFor(context, view_type, obj):
    """Lookup a Silva view, fallback on a Five default one if it
    doesn't exists.
    """

    view_registry = context.service_view_registry
    try:
        # Try first the correct view
        return view_registry.get_view(view_type, obj.meta_type)
    except KeyError:
        # Not found, search default Five one.
        return view_registry.get_view(view_type, getFiveViewNameFor(obj))


# Default content factory

def ContentFactory(content):
    """A factory for Content factories.

    This generates manage_add<Something> for non-versioned content types.
    """
    def factory_method(self, id, title, *args, **kw):
        container = self
        if not mangle.Id(container, id).isValid():
            return
        object = content(id)
        self._setObject(id, object)
        object = getattr(container, id)
        object.set_title(title)
        add_and_edit(container, id, None)
        return ''
    return factory_method

def VersionedContentFactory(extension_name, content, version):
    """A factory for Versioned Content factories.

    This generates manage_add<Something> for versioned content types. It
    makes sure the first version is already added.
    """
    def factory_method(self, id, title, *args, **kw):
        container = self
        if not mangle.Id(container, id).isValid():
            return
        object = content(id)
        container._setObject(id, object)
        object = getattr(container, id)

        version_factory_name = getFactoryName(version)
        extension = extensionRegistry.get_extension(extension_name)

        version_factory = getattr(
            object.manage_addProduct[extension.product],
            version_factory_name)
        version_factory('0', title, *args, **kw)
        object.create_version('0', None, None)
        add_and_edit(container, id, None)
        return ''
    return factory_method

def VersionFactory(version_class):
    """A factory for Version factories.

    This generateas manage_add<Something>Version for versions.
    """
    def factory_method(self, id, title, *args, **kw):
        container = self
        version = version_class(id, *args, **kw)
        container._setObject(id, version)
        version = container._getOb(id)
        version.set_title(title)
        add_and_edit(container, id, None)
        return ''
    return factory_method

# Helpers

def getFactoryDispatcher(product):    
    """Get the Factory Dispatcher, some Zope 2 magic.
    """
    fd = getattr(product, '__FactoryDispatcher__', None)
    if fd is None:
        class __FactoryDispatcher__(FactoryDispatcher):
            "Factory Dispatcher for a Specific Product"
            
        fd = product.__FactoryDispatcher__ = __FactoryDispatcher__
    return fd

def getProductMethods(product):
    """Get the methods dictionary for a product.

    This can be used to register product-level methods, such as factory
    functions like manage_addFoo.
    """
    try:
        return product._m
    except AttributeError:
        fd = getFactoryDispatcher(product)
        product._m = result = AttrDict(fd)
        return result

def getAddPermissionName(class_):
    return 'Add %ss' % class_.meta_type

def getFactoryName(class_):
    return 'manage_add' + class_.__name__


# for use in test cleanup
_register_monkies = []
_meta_type_regs = []

# Registration methods

#visibility can be "Global" or None
def registerClass(class_, extension_name, zmi_addable=False,
                  default_action=None):
    """Register a class with Zope as a type.
    """
    permission = getAddPermissionName(class_)
    interfaces = instancesOfObjectImplements(class_) + list(implementedBy(class_))

    if default_action is None:
        default_action = 'manage_main'

    #There are two ways to remove this object from the list,
    #by either specifying "visibility: none", in which case
    #the object is never visible (and copy support is broken)
    #or by specifying a container_filter, which can return true
    #if the container is an ISilvaObject.  I'm opting for the latter,
    #as at least the objects aren't addable from outside the Silva Root then.
    #Also, Products.Silva.Folder will override ObjectManager.filtered_meta_types
    #to also remove any ISilvaObject types from the add list
    info = {'name': class_.meta_type,
            'action': default_action,
            'product': extension_name,
            'permission': permission,
            'visibility': "Global",
            'interfaces': interfaces,
            'instance': class_,
            'container_filter': makeContainerFilter(zmi_addable)
            }
    Products.meta_types += (info,)

    # register for test cleanup
    _register_monkies.append(class_)
    _meta_type_regs.append(class_.meta_type)

def registerFactory(methods, class_, factory):
    """Register a manage_add<Something> style factory method.
    """
    permission = getAddPermissionName(class_)
    default = ('Manager',)
    AccessControl.Permission.registerPermissions(((permission, (), default),))
    pr = PermissionRole(permission, default)
    if isinstance(factory, tuple) or isinstance(factory, list):
        # just register the factories
        for method in factory:
            name = method.__name__
            if not name.startswith('manage_add'):
                name = getFactoryName(class_)
                module = resolve(class_.__module__)
                setattr(module, name, method)
            methods[name] = method
            methods[name + '__roles__'] = pr
    else:
        # only register this method, and save it in the module with a
        # good name to be imported.
        name = getFactoryName(class_)
        module = resolve(class_.__module__)
        setattr(module, name, factory)
        methods[name] = factory
        methods[name + '__roles__'] = pr

def registerIcon(extension_name, class_, icon):
    """Register icon for a class.
    """
    if not icon:
        return
    
    name = os.path.basename(icon)
    extension = extensionRegistry.get_extension(extension_name)
    icon = Globals.ImageFile(icon, extension.module_directory)
    icon.__roles__ = None
    if not hasattr(icons.misc_, extension_name):
        setattr(icons.misc_, extension_name,
                icons.Misc_(extension_name, {}))
    getattr(icons.misc_, extension_name)[name] = icon
    icon_path = 'misc_/%s/%s' % (extension_name, name)
    
    icon_registry._icon_mapping[('meta_type', class_.meta_type)] = icon_path
    class_.icon = icon_path

def cleanUp():
    global _register_monkies
    for class_ in _register_monkies:
        unregisterClass(class_)
    _register_monkies = []

    global _meta_type_regs
    Products.meta_types = tuple([ info for info in Products.meta_types
                                  if info['name'] not in _meta_type_regs ])
    _meta_type_regs = []

from zope.testing.cleanup import addCleanUp
addCleanUp(cleanUp)
del addCleanUp



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2009 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import zope.cachedescriptors.property
from zope.component.interface import provideInterface
from zope import interface

from Globals import package_home
import os.path

class SystemExtensionInstaller(object):
    """Installer for system extension: there are always installed.
    """

    def install(self, root):
        pass

    def uninstall(self, root):
        pass

    def is_installed(self, root):
        return True


class DefaultInstaller(object):
    """Default installer for extension.
    """


    def __init__(self, name, marker_interface):
        self._name = name
        self._interface = marker_interface
        provideInterface('', self._interface)

    def install(self, root):
        """Default installer.
        """

        if self.is_installed(root):
            return              # Don't install already installed extension
        self.install_custom(root)
        interface.alsoProvides(root.service_extensions, self._interface)

    def install_custom(self, root):
        """Custom installation steps.
        """
        pass

    def uninstall(self, root):
        """Default uninstaller.
        """

        if not self.is_installed(root):
            return              # Don't uninstall uninstalled extension.
        self.uninstall_custom(root)
        interface.noLongerProvides(root.service_extensions, self._interface)

    def uninstall_custom(self, root):
        """Custom uninstall steps.
        """
        pass

    def is_installed(self, root):
        return self._interface.providedBy(root.service_extensions)

    def configureMetadata(self, root, mapping, where=None):
        if where is None:
            where = globals()
        product = package_home(where)
        schema = os.path.join(product, 'schema')
        collection = root.service_metadata.getCollection()

        for types, setids in mapping.items():
            for setid in setids:
                if not setid in collection.objectIds():
                    xmlfile = os.path.join(schema, setid+'.xml')
                    definition = open(xmlfile, 'r')
                    collection.importSet(definition)
            root.service_metadata.addTypesMapping(types, setids)
        root.service_metadata.initializeMetadata()

    def unconfigureMetadata(self, root, mapping):
        all_types = []
        all_sets = []
        for types, setids in mapping.items():
            for typename in types:
                if typename in all_types:
                    continue
                all_types.append(typename)
            for setid in setids:
                if setid in all_sets:
                    continue
                all_sets.append(setid)
        root.service_metadata.removeTypesMapping(all_types, all_sets)
        root.service_metadata.initializeMetadata()
        # delete metadata description if there
        collection = root.service_metadata.getCollection()
        setids = set()
        for values in mapping.values():
            setids.update(values)
        for setid in setids:
            if hasattr(collection, setid):
                # TODO check if setid is still used somewhere
                collection.manage_delObjects([setid,])




# Copyright (c) 2002-2009 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

# Martian directive for contents
from silva.core.conf.martiansupport.directives import *

# Grokcore helpers
from grokcore.component import subscribe
from grokcore import component
from grokcore.view import template, templatedir, require

from zope.interface import implements



# Copyright (c) 2002-2009 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from zope.configuration.interfaces import InvalidToken
from zope.i18n import translate
from zope.interface import implements
from zope.schema import interfaces
from zope import schema, component
from zope.app.intid.interfaces import IIntIds

from Products.Silva import mangle

# the TupleTokens field was created to support multiple values in the
# depends_on attribute of silva:extension.  There is a Tokens field in
# zope.configuration.fields, but it extends from schema.List it
# doesn't support the 'default' parameter, as the default then has to
# be a list, and lists aren't hashable.  TupleTokens supports default
# values

class TupleTokens(schema.Tuple):

    implements(interfaces.IFromUnicode)

    def fromUnicode(self, u):
        u = u.strip()
        if u:
            vt = self.value_type.bind(self.context)
            values = []
            for s in u.split():
                try:
                    v = vt.fromUnicode(s)
                except schema.ValidationError, v:
                    raise InvalidToken("%s in %s" % (v, u))
                else:
                    values.append(v)
            values = tuple(values)
        else:
            values = ()
        self.validate(values)
        return values


from Products.Silva.i18n import translate as _
from zope.i18n import translate

class InvalidID(interfaces.InvalidValue):

    def doc(self):
        value, err_code = self.args

        if err_code == mangle.Id.CONTAINS_BAD_CHARS:
            return _('Sorry, strange characters are in the id. It should only'
                     'contain letters, digits and "_" or "-" or'
                     '"." Spaces are not allowed in Internet addresses,'
                     'and the id should start with a letter or digit.')
        elif err_code == mangle.Id.RESERVED_PREFIX:
            prefix = str(value).split('_')[0]+'_'
            return _("Sorry, ids starting with ${prefix} are reserved for "
                     "internal use. Please use another id.",
                     mapping={'prefix': prefix})
        elif err_code == mangle.Id.RESERVED:
            return _("Sorry, the id ${id} is reserved for internal use. "
                     "Please use another id.", mapping={'id': value})
        elif err_code == mangle.Id.IN_USE_CONTENT:
            return _("There is already an object with the id ${id} in this "
                     "folder. Please use a different one.",
                     mapping={'id': value})
        elif err_code == mangle.Id.IN_USE_ASSET:
            return _("There is already an asset with the id ${id} in this "
                     "folder. Please use another id.",
                     mapping={'id': value})
        elif err_code == mangle.Id.RESERVED_POSTFIX:
            return _("Sorry, the id ${id} ends with invalid characters. Please "
                     "use another id.", mapping={'id': value})
        elif err_code == mangle.Id.IN_USE_ZOPE:
            return _("Sorry, the id ${id} is already in use by a Zope object. "
                     "Please use another id.", mapping={'id': value})
        return _("(Internal Error): An invalid status ${status_code} occured "
                 "while checking the id ${id}. Please contact the person "
                 "responsible for this Silva installation or file a bug report.""",
                 mapping={'status_code': err_code, 'id': value})


class ID(schema.TextLine):
    """Zope 3 schema field for mangle.Id fields.
    """

    def _validate(self, value):
        super(ID, self)._validate(value)
        if self.context:
            mangled = mangle.Id(self.context, value)
            err_code = mangled.validate()
            if err_code != mangled.OK:
                raise InvalidID(value, err_code)
        return


class IContentReference(interfaces.IObject):
    """Field to store a reference to an object
    """

    def fromRelativePath(context, path):
        """Return the value from a relative path.
        """

    def toRelativePath(context, value):
        """Return the value as a relative path.
        """


class ContentReference(schema.Object):
    """Zoep 3 schema field to store a reference to an object.
    """

    implements(IContentReference)

    def fromRelativePath(self, context, path):
        parts = path.split('/')
        if not parts[0]:
            # path start with '/'
            context = context.get_root()
            parts.pop(0)
        return context.restrictedTraverse(parts)

    def toRelativePath(self, context, value):
        return mangle.Path.fromObject(context.getPhysicalPath(), value)


_marker = object()

class ContentReferenceProperty(object):

    def __init__(self, name):
        self.__name = name
        self.__cache_name = '_v_%s' % name

    def __get__(self, inst, klass):
        if inst is None:
            return self

        cache = inst.__dict__.get(self.__cache_name, _marker)
        if cache is not _marker:
            return cache

        iid = inst.__dict__.get(self.__name, None)
        if iid is not None:
            utility = component.getUtility(IIntIds)
            obj =  utility.getObject(iid)
            inst.__dict__[self.__cache_name] = obj
            return obj
        return None

    def __set__(self, inst, value):
        iid = None
        if value is not None:
            utility = component.getUtility(IIntIds)
            iid = utility.register(value)
        inst.__dict__[self.__cache_name] = value
        inst.__dict__[self.__name] = iid
        inst.__dict__['_p_changed'] = True


class IBytes(interfaces.IBytes):
    """Fields which keeps the FileUpload object.
    """

class Bytes(schema.Bytes):
    """See IStreamBytes
    """

    implements(IBytes)

    def validate(self, value):
        # No validation for the time being.
        pass




from setuptools import setup, find_packages
import os

version = '2.1'

setup(name='silva.core.conf',
      version=version,
      description="Configuration machinery for Silva CMS",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
              "Framework :: Zope2",
              "License :: OSI Approved :: BSD License",
              "Programming Language :: Python",
              "Topic :: Software Development :: Libraries :: Python Modules",
              ],
      keywords='silva core configuration',
      author='Infrae',
      author_email='info@infrae.com',
      url='https://github.com/silvacms/silva.core.conf',
      license='BSD',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['silva', 'silva.core'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'five.grok',
          'martian >= 0.11',
          'zope.interface',
          'zope.schema',
          'zope.configuration',
          'zope.testing',
          ],
      )
